import itertools
from pathlib import Path
from typing import Union

import pytest
from lark import Lark, Token, Tree, Visitor

lark_file = Path(__file__).parent.parent / "cn2date/date.lark"
lark_parser = Lark.open(str(lark_file))


class DateTreeVisitorForTest(Visitor):
    def __init__(self):
        self.years_str: Union[str, None] = None
        self.months_str: Union[str, None] = None
        self.days_str: Union[str, None] = None
        self.next_str: Union[str, None] = None
        self.spoken_lang_str: Union[str, None] = None

    def spoken_lang(self, tree: Tree) -> None:
        self.spoken_lang_str = self.__scan_value(tree)

    def years(self, tree: Tree) -> None:
        self.years_str = self.__scan_value(tree)

    def months(self, tree: Tree) -> None:
        self.months_str = self.__scan_value(tree)

    def days(self, tree: Tree) -> None:
        self.days_str = self.__scan_value(tree)

    def next(self, tree: Tree) -> None:
        self.next_str = self.__scan_value(tree)

    @staticmethod
    def __scan_value(tree: Tree) -> str:
        val = ""
        for child in tree.children:
            if not isinstance(child, Token):
                raise TypeError("子节点不是 Token")
            val += child.value
        return val


test_data_dict = {
    "完整日期格式": [
        ("2021/9/10", "2021", "9", "10", None, None),
        ("2021/09/10", "2021", "09", "10", None, None),
        ("2021/09/1", "2021", "09", "1", None, None),
        ("2021/9/1", "2021", "9", "1", None, None),
        ("21/9/10", "21", "9", "10", None, None),
        ("21/09/10", "21", "09", "10", None, None),
        ("21/09/1", "21", "09", "1", None, None),
        ("21/9/1", "21", "9", "1", None, None),
        ("2021-9-10", "2021", "9", "10", None, None),
        ("2021-09-10", "2021", "09", "10", None, None),
        ("2021-09-1", "2021", "09", "1", None, None),
        ("2021-9-1", "2021", "9", "1", None, None),
        ("21-9-10", "21", "9", "10", None, None),
        ("21-09-10", "21", "09", "10", None, None),
        ("21-09-1", "21", "09", "1", None, None),
        ("21-9-1", "21", "9", "1", None, None),
        ("2021年9月10日", "2021", "9", "10", None, None),
        ("2021年09月10日", "2021", "09", "10", None, None),
        ("2021年09月10", "2021", "09", "10", None, None),
        ("2021年09月1", "2021", "09", "1", None, None),
        ("2021年9月1", "2021", "9", "1", None, None),
        ("21年9月10日", "21", "9", "10", None, None),
        ("21年09月10日", "21", "09", "10", None, None),
        ("21年09月10", "21", "09", "10", None, None),
        ("21年09月1", "21", "09", "1", None, None),
        ("21年9月1", "21", "9", "1", None, None),
        ("二零二一/九/三十一", "二零二一", "九", "三十一", None, None),
        ("二零二一/九/十", "二零二一", "九", "十", None, None),
        ("二零二一/零九/十", "二零二一", "零九", "十", None, None),
        ("二零二一/零九/一", "二零二一", "零九", "一", None, None),
        ("二零二一/九/一", "二零二一", "九", "一", None, None),
        ("二一/九/十", "二一", "九", "十", None, None),
        ("二一/零九/十", "二一", "零九", "十", None, None),
        ("二一/零九/一", "二一", "零九", "一", None, None),
        ("二一/九/一", "二一", "九", "一", None, None),
        ("二零二一-九-三十一", "二零二一", "九", "三十一", None, None),
        ("二零二一-九-十", "二零二一", "九", "十", None, None),
        ("二零二一-零九-十", "二零二一", "零九", "十", None, None),
        ("二零二一-零九-一", "二零二一", "零九", "一", None, None),
        ("二零二一-九-一", "二零二一", "九", "一", None, None),
        ("二一-九-十", "二一", "九", "十", None, None),
        ("二一-零九-十", "二一", "零九", "十", None, None),
        ("二一-零九-一", "二一", "零九", "一", None, None),
        ("二一-九-一", "二一", "九", "一", None, None),
        ("二零二一年九月三十一日", "二零二一", "九", "三十一", None, None),
        ("二零二一年九月三十一号", "二零二一", "九", "三十一", None, None),
        ("二零二一年九月三十一", "二零二一", "九", "三十一", None, None),
        ("二零二一年九月十日", "二零二一", "九", "十", None, None),
        ("二零二一年零九月十日", "二零二一", "零九", "十", None, None),
        ("二零二一年零九月十", "二零二一", "零九", "十", None, None),
        ("二零二一年零九月一", "二零二一", "零九", "一", None, None),
        ("二零二一年九月一", "二零二一", "九", "一", None, None),
        ("二一年九月十日", "二一", "九", "十", None, None),
        ("二一年零九月十日", "二一", "零九", "十", None, None),
        ("二一年零九月十", "二一", "零九", "十", None, None),
        ("二一年零九月一", "二一", "零九", "一", None, None),
        ("二一年九月一", "二一", "九", "一", None, None),
        ("二零二一年09月10号", "二零二一", "09", "10", None, None),
        ("二零二一年09/10号", "二零二一", "09", "10", None, None),
        ("二零二一/09月十号", "二零二一", "09", "十", None, None),
        ("二零二一年09-十日", "二零二一", "09", "十", None, None),
        ("二零二一年09-十号", "二零二一", "09", "十", None, None),
        ("二零二一年09-十", "二零二一", "09", "十", None, None),
        ("二零二一年09/10", "二零二一", "09", "10", None, None),
        ("二零二一年09-10", "二零二一", "09", "10", None, None),
        ("2021/九月10", "2021", "九", "10", None, None),
        ("2021/9月十", "2021", "9", "十", None, None),
        ("2021年9月10号", "2021", "9", "10", None, None),
        ("2021年09月10号", "2021", "09", "10", None, None),
        ("21年9月10号", "21", "9", "10", None, None),
        ("21年09月10号", "21", "09", "10", None, None),
        ("二零二一年九月十号", "二零二一", "九", "十", None, None),
        ("二零二一年零九月十号", "二零二一", "零九", "十", None, None),
        ("二一年九月十号", "二一", "九", "十", None, None),
        ("二一年零九月十号", "二一", "零九", "十", None, None),
    ],
    "年月格式": [
        ("2021年09月", "2021", "09", None, None, None),
        ("2021年9月", "2021", "9", None, None, None),
        ("21年09月", "21", "09", None, None, None),
        ("21年9月", "21", "9", None, None, None),
        ("二零二一年零九月", "二零二一", "零九", None, None, None),
        ("二零二一年九月", "二零二一", "九", None, None, None),
        ("二一年零九月", "二一", "零九", None, None, None),
        ("二一年九月", "二一", "九", None, None, None),
    ],
    "月日格式": [
        ("9月10日", None, "9", "10", None, None),
        ("09月10日", None, "09", "10", None, None),
        ("09月1日", None, "09", "1", None, None),
        ("9月1日", None, "9", "1", None, None),
        ("9月10号", None, "9", "10", None, None),
        ("09月10号", None, "09", "10", None, None),
        ("09月1号", None, "09", "1", None, None),
        ("9月1号", None, "9", "1", None, None),
        ("九月十日", None, "九", "十", None, None),
        ("零九月十日", None, "零九", "十", None, None),
        ("零九月一日", None, "零九", "一", None, None),
        ("九月一日", None, "九", "一", None, None),
        ("九月十号", None, "九", "十", None, None),
        ("零九月十号", None, "零九", "十", None, None),
        ("零九月一号", None, "零九", "一", None, None),
        ("九月一号", None, "九", "一", None, None),
    ],
    "年格式": [
        ("2021年", "2021", None, None, None, None),
        ("21年", "21", None, None, None, None),
        ("二零二一年", "二零二一", None, None, None, None),
        ("二一年", "二一", None, None, None, None),
    ],
    "月格式": [
        ("1月", None, "1", None, None, None),
        ("01月", None, "01", None, None, None),
        ("12月", None, "12", None, None, None),
        ("一月", None, "一", None, None, None),
        ("零一月", None, "零一", None, None, None),
        ("十二月", None, "十二", None, None, None),
        ("一二月", None, "一二", None, None, None),
    ],
    "日格式": [
        ("31日", None, None, "31", None, None),
        ("1日", None, None, "1", None, None),
        ("01日", None, None, "01", None, None),
        ("三十一日", None, None, "三十一", None, None),
        ("三一日", None, None, "三一", None, None),
        ("一日", None, None, "一", None, None),
        ("零一日", None, None, "零一", None, None),
    ],
    "口语格式": [
        ("今年", None, None, None, None, "今年"),
        ("本年", None, None, None, None, "本年"),
        ("本年份", None, None, None, None, "本年"),
        ("本年度", None, None, None, None, "本年"),
        ("当前年", None, None, None, None, "当前年"),
        ("当前年份", None, None, None, None, "当前年"),
        ("当前年度", None, None, None, None, "当前年"),
        ("明年", None, None, None, None, "明年"),
        ("去年", None, None, None, None, "去年"),
        ("前年", None, None, None, None, "前年"),
        ("上半年", None, None, None, None, "上半年"),
        ("下半年", None, None, None, None, "下半年"),
        ("前两年", None, None, None, None, "前两年"),
        ("前2年", None, None, None, None, "前2年"),
        ("前两个年", None, None, None, None, "前两年"),
        ("前2个年", None, None, None, None, "前2年"),
        ("前两个年份", None, None, None, None, "前两年"),
        ("前2个年份", None, None, None, None, "前2年"),
        ("两年前", None, None, None, None, "两年前"),
        ("2年前", None, None, None, None, "2年前"),
        ("两年内", None, None, None, None, "两年内"),
        ("2年内", None, None, None, None, "2年内"),
        ("两年以前", None, None, None, None, "两年以前"),
        ("2年以前", None, None, None, None, "2年以前"),
        ("两年之前", None, None, None, None, "两年之前"),
        ("2年之前", None, None, None, None, "2年之前"),
        ("两年以内", None, None, None, None, "两年以内"),
        ("2年以内", None, None, None, None, "2年以内"),
        ("两年之内", None, None, None, None, "两年之内"),
        ("2年之内", None, None, None, None, "2年之内"),
        ("两年以来", None, None, None, None, "两年以来"),
        ("2年以来", None, None, None, None, "2年以来"),
        ("本季度", None, None, None, None, "本季度"),
        ("这个季度", None, None, None, None, "这个季度"),
        ("当前季度", None, None, None, None, "当前季度"),
        ("上季度", None, None, None, None, "上季度"),
        ("上个季度", None, None, None, None, "上季度"),
        ("下季度", None, None, None, None, "下季度"),
        ("下个季度", None, None, None, None, "下季度"),
        ("第一季度", None, None, None, None, "一季度"),
        ("第一个季度", None, None, None, None, "一季度"),
        ("第1季度", None, None, None, None, "1季度"),
        ("第1个季度", None, None, None, None, "1季度"),
        ("一季度", None, None, None, None, "一季度"),
        ("1季度", None, None, None, None, "1季度"),
        ("第二季度", None, None, None, None, "二季度"),
        ("第二个季度", None, None, None, None, "二季度"),
        ("第2季度", None, None, None, None, "2季度"),
        ("第2个季度", None, None, None, None, "2季度"),
        ("2季度", None, None, None, None, "2季度"),
        ("二季度", None, None, None, None, "二季度"),
        ("第三季度", None, None, None, None, "三季度"),
        ("第三个季度", None, None, None, None, "三季度"),
        ("第3季度", None, None, None, None, "3季度"),
        ("第3个季度", None, None, None, None, "3季度"),
        ("三季度", None, None, None, None, "三季度"),
        ("3季度", None, None, None, None, "3季度"),
        ("第四季度", None, None, None, None, "四季度"),
        ("第四个季度", None, None, None, None, "四季度"),
        ("第4季度", None, None, None, None, "4季度"),
        ("第4个季度", None, None, None, None, "4季度"),
        ("四季度", None, None, None, None, "四季度"),
        ("4季度", None, None, None, None, "4季度"),
        ("前两季度", None, None, None, None, "前两季度"),
        ("前2季度", None, None, None, None, "前2季度"),
        ("前两个季度", None, None, None, None, "前两季度"),
        ("前2个季度", None, None, None, None, "前2季度"),
        ("两季度前", None, None, None, None, "两季度前"),
        ("2季度前", None, None, None, None, "2季度前"),
        ("两季度内", None, None, None, None, "两季度内"),
        ("2季度内", None, None, None, None, "2季度内"),
        ("两季度以来", None, None, None, None, "两季度以来"),
        ("2季度以来", None, None, None, None, "2季度以来"),
        ("两个季度前", None, None, None, None, "两季度前"),
        ("2个季度前", None, None, None, None, "2季度前"),
        ("两个季度以前", None, None, None, None, "两季度以前"),
        ("2个季度以前", None, None, None, None, "2季度以前"),
        ("两个季度之前", None, None, None, None, "两季度之前"),
        ("2个季度之前", None, None, None, None, "2季度之前"),
        ("两个季度以内", None, None, None, None, "两季度以内"),
        ("2个季度以内", None, None, None, None, "2季度以内"),
        ("两个季度之内", None, None, None, None, "两季度之内"),
        ("2个季度之内", None, None, None, None, "2季度之内"),
        ("两个季度以来", None, None, None, None, "两季度以来"),
        ("2个季度以来", None, None, None, None, "2季度以来"),
        ("本月", None, None, None, None, "本月"),
        ("本月份", None, None, None, None, "本月"),
        ("本月度", None, None, None, None, "本月"),
        ("这个月", None, None, None, None, "这个月"),
        ("这个月份", None, None, None, None, "这个月"),
        ("当前月", None, None, None, None, "当前月"),
        ("当前月份", None, None, None, None, "当前月"),
        ("上月", None, None, None, None, "上月"),
        ("上月份", None, None, None, None, "上月"),
        ("上个月", None, None, None, None, "上月"),
        ("上个月份", None, None, None, None, "上月"),
        ("下月", None, None, None, None, "下月"),
        ("下月份", None, None, None, None, "下月"),
        ("下个月", None, None, None, None, "下月"),
        ("下个月份", None, None, None, None, "下月"),
        ("两月前", None, None, None, None, "两月前"),
        ("2月前", None, None, None, None, "2月前"),
        ("两月内", None, None, None, None, "两月内"),
        ("2月内", None, None, None, None, "2月内"),
        ("两月以前", None, None, None, None, "两月以前"),
        ("两月之前", None, None, None, None, "两月之前"),
        ("两月以内", None, None, None, None, "两月以内"),
        ("2月以内", None, None, None, None, "2月以内"),
        ("两月之内", None, None, None, None, "两月之内"),
        ("2月之内", None, None, None, None, "2月之内"),
        ("两月以来", None, None, None, None, "两月以来"),
        ("两个月前", None, None, None, None, "两月前"),
        ("2个月前", None, None, None, None, "2月前"),
        ("两个月内", None, None, None, None, "两月内"),
        ("2个月内", None, None, None, None, "2月内"),
        ("两个月以前", None, None, None, None, "两月以前"),
        ("2个月以前", None, None, None, None, "2月以前"),
        ("两个月之前", None, None, None, None, "两月之前"),
        ("2个月之前", None, None, None, None, "2月之前"),
        ("两个月以内", None, None, None, None, "两月以内"),
        ("2个月以内", None, None, None, None, "2月以内"),
        ("两个月之内", None, None, None, None, "两月之内"),
        ("2个月之内", None, None, None, None, "2月之内"),
        ("两个月以来", None, None, None, None, "两月以来"),
        ("2个月以来", None, None, None, None, "2月以来"),
        ("前两月", None, None, None, None, "前两月"),
        ("前2月", None, None, None, None, "前2月"),
        ("前两个月", None, None, None, None, "前两月"),
        ("前2个月", None, None, None, None, "前2月"),
        ("本周", None, None, None, None, "本周"),
        ("当前周", None, None, None, None, "当前周"),
        ("本星期", None, None, None, None, "本星期"),
        ("这个星期", None, None, None, None, "这个星期"),
        ("当前星期", None, None, None, None, "当前星期"),
        ("上周", None, None, None, None, "上周"),
        ("上星期", None, None, None, None, "上星期"),
        ("上个星期", None, None, None, None, "上星期"),
        ("下周", None, None, None, None, "下周"),
        ("下星期", None, None, None, None, "下星期"),
        ("下个星期", None, None, None, None, "下星期"),
        ("前两星期", None, None, None, None, "前两星期"),
        ("前2星期", None, None, None, None, "前2星期"),
        ("前两个星期", None, None, None, None, "前两星期"),
        ("前2个星期", None, None, None, None, "前2星期"),
        ("两星期前", None, None, None, None, "两星期前"),
        ("2星期前", None, None, None, None, "2星期前"),
        ("两星期内", None, None, None, None, "两星期内"),
        ("2星期内", None, None, None, None, "2星期内"),
        ("两星期以来", None, None, None, None, "两星期以来"),
        ("2星期以来", None, None, None, None, "2星期以来"),
        ("两星期以内", None, None, None, None, "两星期以内"),
        ("2星期以内", None, None, None, None, "2星期以内"),
        ("两星期之内", None, None, None, None, "两星期之内"),
        ("2星期之内", None, None, None, None, "2星期之内"),
        ("两星期以来", None, None, None, None, "两星期以来"),
        ("2星期以来", None, None, None, None, "2星期以来"),
        ("两个星期前", None, None, None, None, "两星期前"),
        ("2个星期前", None, None, None, None, "2星期前"),
        ("两个星期以前", None, None, None, None, "两星期以前"),
        ("2个星期以前", None, None, None, None, "2星期以前"),
        ("两个星期之前", None, None, None, None, "两星期之前"),
        ("2个星期之前", None, None, None, None, "2星期之前"),
        ("两个星期内", None, None, None, None, "两星期内"),
        ("2个星期内", None, None, None, None, "2星期内"),
        ("两个星期以内", None, None, None, None, "两星期以内"),
        ("2个星期以内", None, None, None, None, "2星期以内"),
        ("两个星期之内", None, None, None, None, "两星期之内"),
        ("2个星期之内", None, None, None, None, "2星期之内"),
        ("两个星期以来", None, None, None, None, "两星期以来"),
        ("2个星期以来", None, None, None, None, "2星期以来"),
        ("今天", None, None, None, None, "今天"),
        ("今日", None, None, None, None, "今日"),
        ("明天", None, None, None, None, "明天"),
        ("明日", None, None, None, None, "明日"),
        ("后天", None, None, None, None, "后天"),
        ("后日", None, None, None, None, "后日"),
        ("昨天", None, None, None, None, "昨天"),
        ("昨日", None, None, None, None, "昨日"),
        ("前天", None, None, None, None, "前天"),
        ("前日", None, None, None, None, "前日"),
        ("上午", None, None, None, None, "上午"),
        ("下午", None, None, None, None, "下午"),
        ("前两天", None, None, None, None, "前两天"),
        ("前2天", None, None, None, None, "前2天"),
        ("前两个天", None, None, None, None, "前两天"),
        ("前2个天", None, None, None, None, "前2天"),
        ("两天前", None, None, None, None, "两天前"),
        ("2天前", None, None, None, None, "2天前"),
        ("两天内", None, None, None, None, "两天内"),
        ("2天内", None, None, None, None, "2天内"),
        ("两天以来", None, None, None, None, "两天以来"),
        ("2天以来", None, None, None, None, "2天以来"),
        ("两天以内", None, None, None, None, "两天以内"),
        ("2天以内", None, None, None, None, "2天以内"),
        ("两天之内", None, None, None, None, "两天之内"),
        ("2天之内", None, None, None, None, "2天之内"),
        ("两天以来", None, None, None, None, "两天以来"),
        ("2天以来", None, None, None, None, "2天以来"),
        ("两个天前", None, None, None, None, "两天前"),
        ("2个天前", None, None, None, None, "2天前"),
        ("两个天以前", None, None, None, None, "两天以前"),
        ("2个天以前", None, None, None, None, "2天以前"),
        ("两个天之前", None, None, None, None, "两天之前"),
        ("2个天之前", None, None, None, None, "2天之前"),
        ("两个天内", None, None, None, None, "两天内"),
        ("2个天内", None, None, None, None, "2天内"),
        ("两个天以内", None, None, None, None, "两天以内"),
        ("2个天以内", None, None, None, None, "2天以内"),
        ("两个天之内", None, None, None, None, "两天之内"),
        ("2个天之内", None, None, None, None, "2天之内"),
        ("两个天以来", None, None, None, None, "两天以来"),
        ("2个天以来", None, None, None, None, "2天以来"),
        ("前两日", None, None, None, None, "前两日"),
        ("前2日", None, None, None, None, "前2日"),
        ("前两个日", None, None, None, None, "前两日"),
        ("前2个日", None, None, None, None, "前2日"),
        ("两日前", None, None, None, None, "两日前"),
        ("2日前", None, None, None, None, "2日前"),
        ("两日内", None, None, None, None, "两日内"),
        ("2日内", None, None, None, None, "2日内"),
        ("两日以来", None, None, None, None, "两日以来"),
        ("两日以内", None, None, None, None, "两日以内"),
        ("2日以内", None, None, None, None, "2日以内"),
        ("两日之内", None, None, None, None, "两日之内"),
        ("2日之内", None, None, None, None, "2日之内"),
        ("两日以来", None, None, None, None, "两日以来"),
        ("两个日前", None, None, None, None, "两日前"),
        ("2个日前", None, None, None, None, "2日前"),
        ("两个日以前", None, None, None, None, "两日以前"),
        ("2个日以前", None, None, None, None, "2日以前"),
        ("两个日之前", None, None, None, None, "两日之前"),
        ("2个日之前", None, None, None, None, "2日之前"),
        ("两个日内", None, None, None, None, "两日内"),
        ("2个日内", None, None, None, None, "2日内"),
        ("两个日以内", None, None, None, None, "两日以内"),
        ("2个日以内", None, None, None, None, "2日以内"),
        ("两个日之内", None, None, None, None, "两日之内"),
        ("2个日之内", None, None, None, None, "2日之内"),
        ("两个日以来", None, None, None, None, "两日以来"),
        ("2个日以来", None, None, None, None, "2日以来"),
    ],
    "组合日期格式": [
        ("2019年以前", "2019", None, None, "以前", None),
        ("2019年之前", "2019", None, None, "之前", None),
        ("2019年以来", "2019", None, None, "以来", None),
        ("6月以前", None, "6", None, "以前", None),
        ("6月之前", None, "6", None, "之前", None),
        ("6月以来", None, "6", None, "以来", None),
        ("13日以前", None, None, "13", "以前", None),
        ("13日之前", None, None, "13", "之前", None),
        ("13日以来", None, None, "13", "以来", None),
        ("5号以前", None, None, "5", "以前", None),
        ("5号之前", None, None, "5", "之前", None),
        ("5号以来", None, None, "5", "以来", None),
        ("2021年上半年", "2021", None, None, "上半年", None),
        ("2021年第一季度", "2021", None, None, "一季度", None),
        ("2021年9月7日上午", "2021", "9", "7", "上午", None),
    ],
}

test_data = list(itertools.chain.from_iterable(test_data_dict.values()))


@pytest.mark.parametrize(
    "input_str,expected_years,expected_months,expected_days,expected_next,expected_spoken_lang",
    test_data,
    ids=[i[0] for i in test_data],
)
def test_lark_parse(
    input_str: str,
    expected_years: str,
    expected_months: str,
    expected_days: str,
    expected_next: str,
    expected_spoken_lang: str,
):
    tree = lark_parser.parse(input_str)
    visitor = DateTreeVisitorForTest()
    visitor.visit(tree)
    assert visitor.years_str == expected_years
    assert visitor.months_str == expected_months
    assert visitor.days_str == expected_days
    assert visitor.next_str == expected_next
    assert visitor.spoken_lang_str == expected_spoken_lang
